@@ -8,9 +8,6 @@ import tempfile
 import time
 from typing import Any, Dict, List, Mapping, Tuple
 
-import six
-from boto3 import Session
-
 from localstack.constants import (
     AWS_REGION_US_EAST_1,
     DEFAULT_BUCKET_MARKER_LOCAL,
@@ -18,6 +15,7 @@ from localstack.constants import (
     DEFAULT_LAMBDA_CONTAINER_REGISTRY,
     DEFAULT_PORT_EDGE,
     DEFAULT_SERVICE_PORTS,
+    ENV_INTERNAL_TEST_RUN,
     FALSE_STRINGS,
     INSTALL_DIR_INFRA,
     LOCALHOST,
@@ -190,15 +188,6 @@ def load_environment(profile: str = None):
     dotenv.load_dotenv(path, override=False)
 
 
-def has_docker():
-    try:
-        with open(os.devnull, "w") as devnull:
-            subprocess.check_output("docker ps", stderr=devnull, shell=True)
-        return True
-    except Exception:
-        return False
-
-
 def is_linux():
     return platform.system() == "Linux"
 
@@ -381,6 +370,9 @@ EAGER_SERVICE_LOADING = is_env_true("EAGER_SERVICE_LOADING")
 # Whether to skip downloading additional infrastructure components (e.g., custom Elasticsearch versions)
 SKIP_INFRA_DOWNLOADS = os.environ.get("SKIP_INFRA_DOWNLOADS", "").strip()
 
+# Whether to skip downloading our signed SSL cert.
+SKIP_SSL_CERT_DOWNLOAD = is_env_true("SKIP_SSL_CERT_DOWNLOAD")
+
 # whether to enable legacy record&replay persistence mechanism (default true, but will be disabled in a future release!)
 LEGACY_PERSISTENCE = is_env_not_false("LEGACY_PERSISTENCE")
 
@@ -420,14 +412,6 @@ else:
 # additional CLI commands, can be set by plugins
 CLI_COMMANDS = {}
 
-# set of valid regions
-VALID_PARTITIONS = set(Session().get_available_partitions())
-VALID_REGIONS = set()
-for partition in VALID_PARTITIONS:
-    for region in Session().get_available_regions("sns", partition):
-        VALID_REGIONS.add(region)
-
-
 # determine IP of Docker bridge
 if not DOCKER_BRIDGE_IP:
     DOCKER_BRIDGE_IP = "172.17.0.1"
@@ -468,7 +452,7 @@ EXTERNAL_SERVICE_PORTS_START = int(
 EXTERNAL_SERVICE_PORTS_END = int(
     os.environ.get("EXTERNAL_SERVICE_PORTS_END")
     or os.environ.get("SERVICE_INSTANCES_PORTS_END")
-    or (EXTERNAL_SERVICE_PORTS_START + 30)
+    or (EXTERNAL_SERVICE_PORTS_START + 50)
 )
 
 # java options to Lambda
@@ -505,6 +489,9 @@ DYNAMODB_WRITE_ERROR_PROBABILITY = float(
 # JAVA EE heap size for dynamodb
 DYNAMODB_HEAP_SIZE = os.environ.get("DYNAMODB_HEAP_SIZE", "").strip() or "256m"
 
+# single DB instance across multiple credentials are regions
+DYNAMODB_SHARE_DB = int(os.environ.get("DYNAMODB_SHARE_DB") or 0)
+
 # expose SQS on a specific port externally
 SQS_PORT_EXTERNAL = int(os.environ.get("SQS_PORT_EXTERNAL") or 0)
 
@@ -532,6 +519,9 @@ LAMBDA_DOCKER_DNS = os.environ.get("LAMBDA_DOCKER_DNS", "").strip()
 # additional flags passed to Lambda Docker run/create commands
 LAMBDA_DOCKER_FLAGS = os.environ.get("LAMBDA_DOCKER_FLAGS", "").strip()
 
+# prebuild images before execution? Increased cold start time on the tradeoff of increased time until lambda is ACTIVE
+LAMBDA_PREBUILD_IMAGES = is_env_true("LAMBDA_PREBUILD_IMAGES")
+
 # default container registry for lambda execution images
 LAMBDA_CONTAINER_REGISTRY = (
     os.environ.get("LAMBDA_CONTAINER_REGISTRY", "").strip() or DEFAULT_LAMBDA_CONTAINER_REGISTRY
@@ -557,12 +547,8 @@ S3_SKIP_SIGNATURE_VALIDATION = is_env_not_false("S3_SKIP_SIGNATURE_VALIDATION")
 TEST_IAM_USER_ID = str(os.environ.get("TEST_IAM_USER_ID") or "").strip()
 TEST_IAM_USER_NAME = str(os.environ.get("TEST_IAM_USER_NAME") or "").strip()
 
-# whether to use Lambda functions in a Docker container
+# user-defined lambda executor mode
 LAMBDA_EXECUTOR = os.environ.get("LAMBDA_EXECUTOR", "").strip()
-if not LAMBDA_EXECUTOR:
-    LAMBDA_EXECUTOR = "docker"
-    if not has_docker():
-        LAMBDA_EXECUTOR = "local"
 
 # Fallback URL to use when a non-existing Lambda is invoked. If this matches
 # `dynamodb://<table_name>`, then the invocation is recorded in the corresponding
@@ -580,34 +566,38 @@ LAMBDA_CODE_EXTRACT_TIME = int(os.environ.get("LAMBDA_CODE_EXTRACT_TIME") or 25)
 # whether lambdas should use stay open mode if executed in "docker-reuse" executor
 LAMBDA_STAY_OPEN_MODE = is_in_docker and is_env_not_false("LAMBDA_STAY_OPEN_MODE")
 
+# truncate output string slices value
+LAMBDA_TRUNCATE_STDOUT = int(os.getenv("LAMBDA_TRUNCATE_STDOUT") or 2000)
+
 # A comma-delimited string of stream names and its corresponding shard count to
 # initialize during startup.
 # For example: "my-first-stream:1,my-other-stream:2,my-last-stream:1"
 KINESIS_INITIALIZE_STREAMS = os.environ.get("KINESIS_INITIALIZE_STREAMS", "").strip()
 
-# URL to a custom elasticsearch backend cluster. If this is set to a valid URL, then localstack will not create
-# elasticsearch cluster instances, but instead forward all domains to the given backend.
-ES_CUSTOM_BACKEND = os.environ.get("ES_CUSTOM_BACKEND", "").strip()
+# KMS provider - can be either "local-kms" or "moto"
+KMS_PROVIDER = (os.environ.get("KMS_PROVIDER") or "").strip() or "moto"
 
-# Strategy used when creating elasticsearch domain endpoints routed through the edge proxy
-# valid values: domain | path | off
-ES_ENDPOINT_STRATEGY = os.environ.get("ES_ENDPOINT_STRATEGY", "").strip() or "domain"
-
-# Whether to start one cluster per domain (default), or multiplex domains to a single clusters
-ES_MULTI_CLUSTER = is_env_not_false("ES_MULTI_CLUSTER")
-
-# URL to a custom opensearch backend cluster. If this is set to a valid URL, then localstack will not create
-# opensearch cluster instances, but instead forward all domains to the given backend.
-OPENSEARCH_CUSTOM_BACKEND = os.environ.get("OPENSEARCH_CUSTOM_BACKEND", "").strip()
-
-# Strategy used when creating opensearch domain endpoints routed through the edge proxy
-# valid values: domain | path
-OPENSEARCH_ENDPOINT_STRATEGY = (
-    os.environ.get("OPENSEARCH_ENDPOINT_STRATEGY", "").strip() or "domain"
+# URL to a custom OpenSearch/Elasticsearch backend cluster. If this is set to a valid URL, then localstack will not
+# create OpenSearch/Elasticsearch cluster instances, but instead forward all domains to the given backend.
+OPENSEARCH_CUSTOM_BACKEND = (
+    os.environ.get("OPENSEARCH_CUSTOM_BACKEND", "").strip()
+    or os.environ.get("ES_CUSTOM_BACKEND", "").strip()
 )
 
-# Whether to start one openseasrch cluster per domain (default), or multiplex opensearch domains to a single clusters
-OPENSEARCH_MULTI_CLUSTER = is_env_not_false("OPENSEARCH_MULTI_CLUSTER")
+# Strategy used when creating OpenSearch/Elasticsearch domain endpoints routed through the edge proxy
+# valid values: domain | path | port (off)
+OPENSEARCH_ENDPOINT_STRATEGY = (
+    os.environ.get("OPENSEARCH_ENDPOINT_STRATEGY", "").strip()
+    or os.environ.get("ES_ENDPOINT_STRATEGY", "").strip()
+    or "domain"
+)
+if OPENSEARCH_ENDPOINT_STRATEGY == "off":
+    OPENSEARCH_ENDPOINT_STRATEGY = "port"
+
+# Whether to start one cluster per domain (default), or multiplex opensearch domains to a single clusters
+OPENSEARCH_MULTI_CLUSTER = is_env_not_false("OPENSEARCH_MULTI_CLUSTER") or is_env_true(
+    "ES_MULTI_CLUSTER"
+)
 
 # list of environment variable names used for configuration.
 # Make sure to keep this in sync with the above!
@@ -625,6 +615,7 @@ CONFIG_ENV_VARS = [
     "DOCKER_BRIDGE_IP",
     "DYNAMODB_ERROR_PROBABILITY",
     "DYNAMODB_HEAP_SIZE",
+    "DYNAMODB_SHARE_DB",
     "DYNAMODB_READ_ERROR_PROBABILITY",
     "DYNAMODB_WRITE_ERROR_PROBABILITY",
     "EAGER_SERVICE_LOADING",
@@ -656,6 +647,7 @@ CONFIG_ENV_VARS = [
     "LAMBDA_REMOTE_DOCKER",
     "LAMBDA_REMOVE_CONTAINERS",
     "LAMBDA_STAY_OPEN_MODE",
+    "LAMBDA_TRUNCATE_STDOUT",
     "LEGACY_DOCKER_CLIENT",
     "LOCALSTACK_API_KEY",
     "LOCALSTACK_HOSTNAME",
@@ -669,6 +661,7 @@ CONFIG_ENV_VARS = [
     "S3_SKIP_SIGNATURE_VALIDATION",
     "SERVICES",
     "SKIP_INFRA_DOWNLOADS",
+    "SKIP_SSL_CERT_DOWNLOAD",
     "SQS_PORT_EXTERNAL",
     "STEPFUNCTIONS_LAMBDA_ENDPOINT",
     "SYNCHRONOUS_API_GATEWAY_EVENTS",
@@ -691,6 +684,11 @@ CONFIG_ENV_VARS = [
 ]
 
 
+def is_local_test_mode() -> bool:
+    """Returns True if we are running in the context of our local integration tests."""
+    return is_env_true(ENV_INTERNAL_TEST_RUN)
+
+
 def collect_config_items() -> List[Tuple[str, Any]]:
     """Returns a list of key-value tuples of LocalStack configuration values."""
     none = object()  # sentinel object
@@ -754,7 +752,7 @@ SERVICE_PORTS = parse_service_ports()
 def populate_config_env_var_names():
     global CONFIG_ENV_VARS
 
-    for key, value in six.iteritems(DEFAULT_SERVICE_PORTS):
+    for key, value in DEFAULT_SERVICE_PORTS.items():
         clean_key = key.upper().replace("-", "_")
         CONFIG_ENV_VARS += [
             clean_key + "_BACKEND",
@@ -815,6 +813,15 @@ def get_edge_url(localstack_hostname=None, protocol=None):
     return "%s://%s:%s" % (protocol, localstack_hostname, port)
 
 
+def edge_ports_info():
+    if EDGE_PORT_HTTP:
+        result = "ports %s/%s" % (EDGE_PORT, EDGE_PORT_HTTP)
+    else:
+        result = "port %s" % EDGE_PORT
+    result = "%s %s" % (get_protocol(), result)
+    return result
+
+
 # set log levels
 if DEBUG:
     logging.getLogger("").setLevel(logging.DEBUG)
@@ -831,11 +838,19 @@ if LS_LOG in TRACE_LOG_LEVELS:
 class ServiceProviderConfig(Mapping[str, str]):
     _provider_config: Dict[str, str]
     default_value: str
+    override_prefix: str = "PROVIDER_OVERRIDE_"
 
     def __init__(self, default_value: str):
         self._provider_config = {}
         self.default_value = default_value
 
+    def load_from_environment(self, env: Mapping[str, str] = None):
+        if env is None:
+            env = os.environ
+        for key, value in env.items():
+            if key.startswith(self.override_prefix):
+                self.set_provider(key[len(self.override_prefix) :].lower().replace("_", "-"), value)
+
     def get_provider(self, service: str) -> str:
         return self._provider_config.get(service, self.default_value)
 
@@ -865,11 +880,7 @@ class ServiceProviderConfig(Mapping[str, str]):
 
 SERVICE_PROVIDER_CONFIG = ServiceProviderConfig("default")
 
-for key, value in os.environ.items():
-    if key.startswith("PROVIDER_OVERRIDE_"):
-        SERVICE_PROVIDER_CONFIG.set_provider(
-            key.lstrip("PROVIDER_OVERRIDE_").lower().replace("_", "-"), value
-        )
+SERVICE_PROVIDER_CONFIG.load_from_environment()
 
 # initialize directories
 if is_in_docker:
